package Bio::MUST::Drivers::Blast;
# ABSTRACT: Bio::MUST driver for running the NCBI-BLAST+ suite of programs
$Bio::MUST::Drivers::Blast::VERSION = '0.242720';
use strict;
use warnings;

use Bio::MUST::Drivers::Blast::Database;
use Bio::MUST::Drivers::Blast::Database::Temporary;

1;

__END__

=pod

=head1 NAME

Bio::MUST::Drivers::Blast - Bio::MUST driver for running the NCBI-BLAST+ suite of programs

=head1 VERSION

version 0.242720

=head1 SYNOPSIS

    # TODO

=head1 DESCRIPTION

    # TODO

=head1 AUTHOR

Denis BAURAIN <denis.baurain@uliege.be>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2013 by University of Liege / Unit of Eukaryotic Phylogenomics / Denis BAURAIN.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
