package Bio::MUST::Provision::Exonerate;
# ABSTRACT: Internal class for app provisioning system
$Bio::MUST::Provision::Exonerate::VERSION = '0.242720';
# AUTOGENERATED CODE! DO NOT MODIFY THIS FILE!

use Modern::Perl '2011';
use Carp;

use parent qw(App::Provision::Tiny);


sub deps { return qw(brew) }

sub condition {
    my $self = shift;

    my $condition = qx{which exonerate} =~ m/exonerate$/xms;
    carp '[BMD] Note: Exonerate executable not found; I can try brewing it.'
        unless $condition;

    return $condition ? 1 : 0;
}

sub meet {
    my $self = shift;

    return $self->recipe(
        ['brew tap brewsci/bio'],
        ['brew install https://raw.githubusercontent.com/brewsci/homebrew-bio/f68615713d23ab36d979f8cf06268808ff3dd9dc/Formula/exonerate.rb'],
    );
}

1;

__END__

=pod

=head1 NAME

Bio::MUST::Provision::Exonerate - Internal class for app provisioning system

=head1 VERSION

version 0.242720

=head1 AUTHOR

Denis BAURAIN <denis.baurain@uliege.be>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2013 by University of Liege / Unit of Eukaryotic Phylogenomics / Denis BAURAIN.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
